/*
Fast Gauss Bilateral Filtering C++ code 

 Shin Yoshizawa (shin@riken.jp)

Copyright:(c) Shin Yoshizawa, 2014.


 All right is reserved by Shin Yoshizawa.
This C++ code files are allowed for only primary user of research and educational purposes. Don't use secondary: copy, distribution, diversion, business purpose, and etc.. 

 In no event shall the author be liable to any party for direct, indirect, 
special, incidental, or consequential damage arising out of the use of this 
program and source files. 

 The C++ code and its simple manual are available at 
http://www.riken.jp/brict/Yoshizawa/Research/FBGFilter.html
*/

#ifndef SIMPLEIMAGE_H
#define SIMPLEIMAGE_H
//2D Image class
class Image{
 public:
  int sx,sy;
  double px,py;
  double **img;
  int graylevel;
  double maxgray;
  Image (){
    img=NULL;
    sx=sy=0;
    px=py=1.0;
    graylevel=0;maxgray=0.0;
  }
  Image(int dx,int dy){
    sx = dx;
    sy = dy;
    px=py=1.0;maxgray=0.0;
    graylevel=0;
    int i;
    img = new double* [sy];
    for(i=0;i<sy;i++){
      img[i] = new double[dx];
    }
  }
  void setMaxMin(double *dmax,double *dmin){
    (*dmax)=img[0][0];
    (*dmin)=img[0][0];
    
    int i,j;
    for(i=0;i<sy;i++)
      for(j=0;j<sx;j++){
	if((*dmax)<img[i][j])(*dmax)=img[i][j];
	if((*dmin)>img[i][j])(*dmin)=img[i][j];
      }
  }
  void Copy(Image *in){
    int i,j;
    for(i=0;i<sy;i++)
      for(j=0;j<sx;j++)img[i][j] = in->img[i][j];
    graylevel = in->graylevel;
    maxgray = in->maxgray;
    
  }
  
  void setMaxgray(){
    int i,j;maxgray=0.0;
    for(i=0;i<sy;i++)
      for(j=0;j<sx;j++){
	if(maxgray<img[i][j])maxgray = img[i][j];
      }
    if(maxgray<=0.0)maxgray=1.0;
  }
  virtual ~Image(){
    int i;
    for(i=0;i<sy;i++){
      delete [] img[i];
    }
    delete [] img;
  }
};

//3D Image class
class Image3D{
 public:
  int sx,sy,sz;
  double ***img;
  int graylevel;
  double px,py,pz;
  double maxgray;
  Image3D(){
    graylevel=0;
    px=py=pz=1.0;
    img=NULL;
    sx=sy=sz=0;maxgray=0.0;
  }
  Image3D(int dx,int dy,int dz){
    sx = dx;
    sy = dy;
    sz = dz;
    graylevel=0;maxgray=0.0;
    px=py=pz=1.0;
    int i,j;
    img = new double** [sz];
    for(i=0;i<sz;i++){
      img[i] = new double* [dy];
      for(j=0;j<sy;j++)img[i][j] = new double[sx]; 
    }
  }
  Image3D(Image3D *in){
    sx = in->sx;
    sy = in->sy;
    sz = in->sz;
    graylevel=0;maxgray=0.0;
    px=in->px;
    py=in->py;
    pz=in->pz;
    int i,j;
    img = new double** [sz];
    for(i=0;i<sz;i++){
      img[i] = new double* [sy];
      for(j=0;j<sy;j++)img[i][j] = new double[sx]; 
    }
  }
  
  void setMaxMin(double *dmax,double *dmin){
    (*dmax)=img[0][0][0];
    (*dmin)=img[0][0][0];
    
    int i,j,k;
    for(i=0;i<sz;i++)
      for(j=0;j<sy;j++)
	for(k=0;k<sx;k++){
	  if((*dmax)<img[i][j][k])(*dmax)=img[i][j][k];
	  if((*dmin)>img[i][j][k])(*dmin)=img[i][j][k];
	}
      
  }
  void Copy(Image3D *in){
    int i,j,k;
    for(i=0;i<sz;i++)
      for(j=0;j<sy;j++)
	for(k=0;k<sx;k++)img[i][j][k] = in->img[i][j][k];
    graylevel = in->graylevel;
    maxgray = in->maxgray;
    
  }
  
  void setMaxgray(){
    int i,j,k;
    maxgray=0.0;
    for(i=0;i<sz;i++)
      for(j=0;j<sy;j++)
	for(k=0;k<sx;k++){
	  if(maxgray<img[i][j][k])maxgray = img[i][j][k];
      }
    if(maxgray<=0.0)maxgray=1.0;
  }

  virtual ~Image3D(){
    int i,j;
    for(i=0;i<sz;i++){
      for(j=0;j<sy;j++)
	delete [] img[i][j];
      delete [] img[i];
    }
    delete [] img;
  }
};
//4D Image class
class Image4D{
 public:
  int sx,sy,sz,st;
  double ****img;
  int graylevel;
  double px,py,pz,pt;
  double maxgray;
  Image4D(){
    graylevel=0;
    px=py=pz=pt=1.0;
    img=NULL;
    sx=sy=sz=st=0;maxgray=0.0;
  }
  Image4D(int dx,int dy,int dz,int dt){
    sx = dx;
    sy = dy;
    sz = dz;
    st = dt;
    graylevel=0;maxgray=0.0;
    px=py=pz=pt=1.0;
    int i,j,k;
    img = new double*** [st];
    for(i=0;i<st;i++){
      img[i] = new double **[sz];
      for(j=0;j<sz;j++){
	img[i][j] = new double* [dy];
	for(k=0;k<sy;k++)img[i][j][k] = new double[sx]; 
      }
    }
  }
  
  virtual ~Image4D(){
    int i,j,k;
    for(i=0;i<st;i++){
      for(j=0;j<sz;j++){
	for(k=0;k<sy;k++)	
	  delete [] img[i][j][k];
	delete [] img[i][j];
      }
      delete [] img[i];
    }
    delete [] img;
  }
};


#endif
